@@ -33,7 +33,6 @@
 #include "OgreGpuProgramManager.h"
 #include "OgreGLES2RenderSystem.h"
 #include "OgreGLUniformCache.h"
-#include "OgreGLES2HardwareUniformBuffer.h"
 #include "OgreHardwareBufferManager.h"
 #include "OgreGLUtil.h"
 #include "OgreRoot.h"
@@ -41,8 +40,8 @@
 
 namespace Ogre
 {
-    GLSLESProgramPipeline::GLSLESProgramPipeline(GLSLESProgram* vertexProgram, GLSLESProgram* fragmentProgram) :
-    GLSLESProgramCommon(vertexProgram, fragmentProgram) { }
+    GLSLESProgramPipeline::GLSLESProgramPipeline(const GLShaderList& shaders) :
+    GLSLESProgramCommon(shaders) { }
 
     GLSLESProgramPipeline::~GLSLESProgramPipeline()
     {
@@ -51,99 +50,39 @@ namespace Ogre
 
     void GLSLESProgramPipeline::compileAndLink()
     {
-        GLint linkStatus = 0;
-        
         OGRE_CHECK_GL_ERROR(glGenProgramPipelinesEXT(1, &mGLProgramHandle));
         //OGRE_CHECK_GL_ERROR(glBindProgramPipelineEXT(mGLProgramHandle));
 
-        uint32 vhash = 0, fhash = 0;
-
+        mLinked = true;
         // Compile and attach Vertex Program
-        if(getVertexProgram())
+        if(mShaders[GPT_VERTEX_PROGRAM])
         {
-            vhash = mVertexShader->_getHash();
-            if(getVertexProgram()->isLinked())
-            {
-                mLinked |= VERTEX_PROGRAM_LINKED;
-            }
-            else if (getMicrocodeFromCache(vhash, getVertexProgram()->createGLProgramHandle()))
-            {
-                getVertexProgram()->setLinked(true);
-                mLinked |= VERTEX_PROGRAM_LINKED;
-            }
-            else
-            {
-                GLuint programHandle = getVertexProgram()->getGLProgramHandle();
-
-                bindFixedAttributes( programHandle );
-
-                OGRE_CHECK_GL_ERROR(glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE));
-                getVertexProgram()->attachToProgramObject(programHandle);
-                OGRE_CHECK_GL_ERROR(glLinkProgram(programHandle));
-                OGRE_CHECK_GL_ERROR(glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus));
-                
-                if(linkStatus)
-                {
-                    getVertexProgram()->setLinked(linkStatus);
-                    mLinked |= VERTEX_PROGRAM_LINKED;
-                }
-
-                GLSLES::logObjectInfo( getCombinedName() + String("GLSL vertex program result : "), programHandle );
-
-                setSkeletalAnimationIncluded(getVertexProgram()->isSkeletalAnimationIncluded());
-            }
+            mLinked = mLinked && mShaders[GPT_VERTEX_PROGRAM]->linkSeparable();
         }
         
         // Compile and attach Fragment Program
-        if(mFragmentProgram)
+        if(mShaders[GPT_FRAGMENT_PROGRAM])
         {
-            fhash = mFragmentProgram->_getHash();
-            if(mFragmentProgram->isLinked())
-            {
-                mLinked |= FRAGMENT_PROGRAM_LINKED;
-            }
-            else if (getMicrocodeFromCache(fhash, mFragmentProgram->createGLProgramHandle()))
-            {
-                mFragmentProgram->setLinked(true);
-                mLinked |= FRAGMENT_PROGRAM_LINKED;
-            }
-            else
-            {
-                GLuint programHandle = mFragmentProgram->getGLProgramHandle();
-                OGRE_CHECK_GL_ERROR(glProgramParameteriEXT(programHandle, GL_PROGRAM_SEPARABLE_EXT, GL_TRUE));
-                mFragmentProgram->attachToProgramObject(programHandle);
-                OGRE_CHECK_GL_ERROR(glLinkProgram(programHandle));
-                OGRE_CHECK_GL_ERROR(glGetProgramiv(programHandle, GL_LINK_STATUS, &linkStatus));
-
-                if(linkStatus)
-                {
-                    mFragmentProgram->setLinked(linkStatus);
-                    mLinked |= FRAGMENT_PROGRAM_LINKED;
-                }
-
-                GLSLES::logObjectInfo( getCombinedName() + String("GLSL fragment program result : "), programHandle );
-            }
+            mLinked = mLinked && mShaders[GPT_FRAGMENT_PROGRAM]->linkSeparable();
         }
         
         if(mLinked)
         {
-            if(getVertexProgram() && getVertexProgram()->isLinked())
+            if(mShaders[GPT_VERTEX_PROGRAM])
             {
-                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramHandle, GL_VERTEX_SHADER_BIT_EXT, getVertexProgram()->getGLProgramHandle()));
-                _writeToCache(vhash, getVertexProgram()->getGLProgramHandle());
+                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramHandle, GL_VERTEX_SHADER_BIT_EXT, mShaders[GPT_VERTEX_PROGRAM]->getGLProgramHandle()));
             }
-            if(mFragmentProgram && mFragmentProgram->isLinked())
+            if(mShaders[GPT_FRAGMENT_PROGRAM])
             {
-                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramHandle, GL_FRAGMENT_SHADER_BIT_EXT, mFragmentProgram->getGLProgramHandle()));
-                _writeToCache(fhash, mFragmentProgram->getGLProgramHandle());
+                OGRE_CHECK_GL_ERROR(glUseProgramStagesEXT(mGLProgramHandle, GL_FRAGMENT_SHADER_BIT_EXT, mShaders[GPT_FRAGMENT_PROGRAM]->getGLProgramHandle()));
             }
 
             // Validate pipeline
             GLSLES::logObjectInfo( getCombinedName() + String("GLSL program pipeline result : "), mGLProgramHandle );
-            if(getVertexProgram() && mFragmentProgram && Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_DEBUG))
+            if(mShaders[GPT_VERTEX_PROGRAM] && mShaders[GPT_FRAGMENT_PROGRAM] && Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_DEBUG))
             {
                 glLabelObjectEXT(GL_PROGRAM_PIPELINE_OBJECT_EXT, mGLProgramHandle, 0,
-                             (getVertexProgram()->getName() + "/" + mFragmentProgram->getName()).c_str());
+                             (mShaders[GPT_VERTEX_PROGRAM]->getName() + "/" + mShaders[GPT_FRAGMENT_PROGRAM]->getName()).c_str());
             }
         }
     }
@@ -204,19 +143,17 @@ namespace Ogre
         {
             const GpuConstantDefinitionMap* vertParams = 0;
             const GpuConstantDefinitionMap* fragParams = 0;
-            if (getVertexProgram())
+            if (mShaders[GPT_VERTEX_PROGRAM])
             {
-                vertParams = &(getVertexProgram()->getConstantDefinitions().map);
-                GLSLESProgramManager::extractUniforms(getVertexProgram()->getGLProgramHandle(),
-                                                      vertParams, NULL, mGLUniformReferences,
-                                                      mSharedParamsBufferMap);
+                vertParams = &(mShaders[GPT_VERTEX_PROGRAM]->getConstantDefinitions().map);
+                GLSLESProgramManager::extractUniforms(mShaders[GPT_VERTEX_PROGRAM]->getGLProgramHandle(),
+                                                      vertParams, NULL, mGLUniformReferences);
             }
-            if (mFragmentProgram)
+            if (mShaders[GPT_FRAGMENT_PROGRAM])
             {
-                fragParams = &(mFragmentProgram->getConstantDefinitions().map);
-                GLSLESProgramManager::extractUniforms(mFragmentProgram->getGLProgramHandle(), NULL,
-                                                      fragParams, mGLUniformReferences,
-                                                      mSharedParamsBufferMap);
+                fragParams = &(mShaders[GPT_FRAGMENT_PROGRAM]->getConstantDefinitions().map);
+                GLSLESProgramManager::extractUniforms(mShaders[GPT_FRAGMENT_PROGRAM]->getGLProgramHandle(), NULL,
+                                                      fragParams, mGLUniformReferences);
             }
 
             mUniformRefsBuilt = true;
@@ -227,26 +164,13 @@ namespace Ogre
     void GLSLESProgramPipeline::updateUniforms(GpuProgramParametersSharedPtr params, 
                                            uint16 mask, GpuProgramType fromProgType)
     {
+        OgreAssert(mShaders[fromProgType], "invalid program type");
+        GLuint progID = mShaders[fromProgType]->getGLProgramHandle();
+        GLUniformCache* uniformCache = mShaders[fromProgType]->getUniformCache();
+
         // Iterate through uniform reference list and update uniform values
         GLUniformReferenceIterator currentUniform = mGLUniformReferences.begin();
         GLUniformReferenceIterator endUniform = mGLUniformReferences.end();
-        GLuint progID = 0;
-        GLUniformCache* uniformCache=0;
-        if(fromProgType == GPT_VERTEX_PROGRAM && getVertexProgram())
-        {
-            progID = getVertexProgram()->getGLProgramHandle();
-            uniformCache = getVertexProgram()->getUniformCache();
-        }
-        else if(fromProgType == GPT_FRAGMENT_PROGRAM && mFragmentProgram)
-        {
-            progID = mFragmentProgram->getGLProgramHandle();
-            uniformCache = mFragmentProgram->getUniformCache();
-        }
-        else
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "invalid program type");
-        }
-
         for (;currentUniform != endUniform; ++currentUniform)
         {
             // Only pull values from buffer it's supposed to be in (vertex or fragment)
@@ -258,31 +182,11 @@ namespace Ogre
                 if (def->variability & mask)
                 {
                     GLsizei glArraySize = (GLsizei)def->arraySize;
-                    bool shouldUpdate = true;
-                    switch (def->constType)
-                    {
-                        case GCT_INT1:
-                        case GCT_INT2:
-                        case GCT_INT3:
-                        case GCT_INT4:
-                        case GCT_SAMPLER1D:
-                        case GCT_SAMPLER1DSHADOW:
-                        case GCT_SAMPLER2D:
-                        case GCT_SAMPLER2DSHADOW:
-                        case GCT_SAMPLER3D:
-                        case GCT_SAMPLERCUBE:
-                        case GCT_SAMPLER2DARRAY:
-                            shouldUpdate = uniformCache->updateUniform(currentUniform->mLocation,
-                                                                        params->getIntPointer(def->physicalIndex),
-                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(int)));
-                            break;
-                        default:
-                            shouldUpdate = uniformCache->updateUniform(currentUniform->mLocation,
-                                                                        params->getFloatPointer(def->physicalIndex),
-                                                                        static_cast<GLsizei>(def->elementSize * def->arraySize * sizeof(float)));
-                            break;
-                    }
+                    void* val = def->isSampler() ? (void*)params->getRegPointer(def->physicalIndex)
+                                                 : (void*)params->getFloatPointer(def->physicalIndex);
 
+                    bool shouldUpdate = uniformCache->updateUniform(currentUniform->mLocation, val,
+                                                                    def->elementSize * def->arraySize * 4);
                     if(!shouldUpdate)
                         continue;
 
@@ -317,9 +221,17 @@ namespace Ogre
                             OGRE_CHECK_GL_ERROR(glProgramUniformMatrix4fvEXT(progID, currentUniform->mLocation, glArraySize, 
                                                                              GL_FALSE, params->getFloatPointer(def->physicalIndex)));
                             break;
+                        case GCT_SAMPLER1D:
+                        case GCT_SAMPLER1DSHADOW:
+                        case GCT_SAMPLER2D:
+                        case GCT_SAMPLER2DSHADOW:
+                        case GCT_SAMPLER3D:
+                        case GCT_SAMPLERCUBE:
+                        case GCT_SAMPLER2DARRAY:
+                            // Samplers handled like 1-element ints
                         case GCT_INT1:
                             OGRE_CHECK_GL_ERROR(glProgramUniform1ivEXT(progID, currentUniform->mLocation, glArraySize, 
-                                                                       params->getIntPointer(def->physicalIndex)));
+                                                                       (int*)val));
                             break;
                         case GCT_INT2:
                             OGRE_CHECK_GL_ERROR(glProgramUniform2ivEXT(progID, currentUniform->mLocation, glArraySize, 
@@ -333,17 +245,6 @@ namespace Ogre
                             OGRE_CHECK_GL_ERROR(glProgramUniform4ivEXT(progID, currentUniform->mLocation, glArraySize, 
                                                                        params->getIntPointer(def->physicalIndex)));
                             break;
-                        case GCT_SAMPLER1D:
-                        case GCT_SAMPLER1DSHADOW:
-                        case GCT_SAMPLER2D:
-                        case GCT_SAMPLER2DSHADOW:
-                        case GCT_SAMPLER3D:
-                        case GCT_SAMPLERCUBE:
-                        case GCT_SAMPLER2DARRAY:
-                            // Samplers handled like 1-element ints
-                            OGRE_CHECK_GL_ERROR(glProgramUniform1ivEXT(progID, currentUniform->mLocation, 1,
-                                                                       params->getIntPointer(def->physicalIndex)));
-                            break;
                         case GCT_MATRIX_2X3:
                             OGRE_CHECK_GL_ERROR(glProgramUniformMatrix2x3fvEXT(progID, currentUniform->mLocation, glArraySize,
                                                                             GL_FALSE, params->getFloatPointer(def->physicalIndex)));
@@ -369,12 +270,11 @@ namespace Ogre
                                                                             GL_FALSE, params->getFloatPointer(def->physicalIndex)));
                             break;
                         case GCT_UNKNOWN:
-                        case GCT_SUBROUTINE:
+                        case GCT_SPECIALIZATION:
                         case GCT_DOUBLE1:
                         case GCT_DOUBLE2:
                         case GCT_DOUBLE3:
                         case GCT_DOUBLE4:
-                        case GCT_SAMPLERRECT:
                         case GCT_MATRIX_DOUBLE_2X2:
                         case GCT_MATRIX_DOUBLE_2X3:
                         case GCT_MATRIX_DOUBLE_2X4:
