@@ -681,22 +681,6 @@ void Sample_NewInstancing::sliderMoved( Slider* slider )
     NUM_INST_COLUMN = static_cast<int>(mInstancesSlider->getValue());
 }
 
-//------------------------------------------------------------------------------
-void Sample_NewInstancing::testCapabilities( const RenderSystemCapabilities* caps )
-{
-    if (!GpuProgramManager::getSingleton().isSyntaxSupported("glsl") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("glsl300es") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("fp40") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("ps_2_0") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("ps_3_0") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("ps_4_0") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("ps_4_1") &&
-        !GpuProgramManager::getSingleton().isSyntaxSupported("ps_5_0"))
-    {
-        OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "Your card does not support the shader model needed for this sample, "
-                    "so you cannot run this sample. Sorry!", "NewInstancing::testCapabilities");
-    }
-}
 //------------------------------------------------------------------------------
 
 void Sample_NewInstancing::checkHardwareSupport()
