@@ -107,12 +107,7 @@ static void checkCaps(const Ogre::RenderSystemCapabilities* caps)
     EXPECT_EQ(caps->getNumMultiRenderTargets(), (Ogre::ushort)4);
 
     EXPECT_EQ(caps->getVertexProgramConstantFloatCount(), (Ogre::ushort)256);
-    EXPECT_EQ(caps->getVertexProgramConstantIntCount(), (Ogre::ushort)0);
-    EXPECT_EQ(caps->getVertexProgramConstantBoolCount(), (Ogre::ushort)0);
-
     EXPECT_EQ(caps->getFragmentProgramConstantFloatCount(), (Ogre::ushort)64);
-    EXPECT_EQ(caps->getFragmentProgramConstantIntCount(), (Ogre::ushort)0);
-    EXPECT_EQ(caps->getFragmentProgramConstantBoolCount(), (Ogre::ushort)0);
 
     EXPECT_EQ(caps->getNumVertexTextureUnits(), (Ogre::ushort)0);
     EXPECT_TRUE(caps->isShaderProfileSupported("arbvp1"));
